Module BezierStuff
    ' Parametric functions for drawing a degree 3 Bezier curve.
    Private Function X(ByVal t As Single, ByVal x0 As Single, ByVal x1 As Single, ByVal x2 As Single, ByVal x3 As Single) As Single
        X = CSng( _
            x0 * (1 - t) ^ 3 + _
            x1 * 3 * t * (1 - t) ^ 2 + _
            x2 * 3 * t ^ 2 * (1 - t) + _
            x3 * t ^ 3 _
            )
    End Function
    Private Function Y(ByVal t As Single, ByVal y0 As Single, ByVal y1 As Single, ByVal y2 As Single, ByVal y3 As Single) As Single
        Y = CSng( _
            y0 * (1 - t) ^ 3 + _
            y1 * 3 * t * (1 - t) ^ 2 + _
            y2 * 3 * t ^ 2 * (1 - t) + _
            y3 * t ^ 3 _
            )
    End Function

    ' Draw the Bezier curve.
    Public Sub DrawBezier(ByVal gr As Graphics, ByVal the_pen As Pen, ByVal dt As Single, ByVal pt0 As PointF, ByVal pt1 As PointF, ByVal pt2 As PointF, ByVal pt3 As PointF)
        ' Debugging code commented out.
        ' Draw the control lines.
        Dim dashed_pen As New Pen(Color.Black)
        dashed_pen.DashStyle = Drawing2D.DashStyle.Custom
        dashed_pen.DashPattern = New Single() {4, 4}
        gr.DrawLine(dashed_pen, pt0.X, pt0.Y, pt1.X, pt1.Y)
        gr.DrawLine(dashed_pen, pt2.X, pt2.Y, pt3.X, pt3.Y)
        dashed_pen.Dispose()

        ' Draw the curve.
        Dim t, x0, y0, x1, y1 As Single

        t = 0.0
        x1 = X(t, pt0.X, pt1.X, pt2.X, pt3.X)
        y1 = Y(t, pt0.Y, pt1.Y, pt2.Y, pt3.Y)
        t += dt
        Do While t < 1.0
            x0 = x1
            y0 = y1
            x1 = X(t, pt0.X, pt1.X, pt2.X, pt3.X)
            y1 = Y(t, pt0.Y, pt1.Y, pt2.Y, pt3.Y)
            gr.DrawLine(the_pen, x0, y0, x1, y1)
            t += dt
        Loop

        ' Connect to the final point.
        t = 1.0
        x0 = x1
        y0 = y1
        x1 = X(t, pt0.X, pt1.X, pt2.X, pt3.X)
        y1 = Y(t, pt0.Y, pt1.Y, pt2.Y, pt3.Y)
        gr.DrawLine(the_pen, x0, y0, x1, y1)
    End Sub

    ' Draw a closed cardinal spline built from connected Bezier curves.
    Public Sub DrawClosedCurve(ByVal gr As Graphics, ByVal the_pen As Pen, ByVal dt As Single, ByVal tension As Single, ByVal pts() As PointF)
        Dim control_scale As Single = CSng(tension / 0.5 * 0.175)
        Dim pt, pt_before, pt_after, pt_after2, Di, DiPlus1 As PointF
        Dim p1, p2, p3, p4 As PointF
        Dim num_pts As Integer = pts.Length
        For i As Integer = 0 To pts.GetUpperBound(0)
            pt_before = pts((i - 1 + num_pts) Mod num_pts)
            pt = pts(i)
            pt_after = pts((i + 1) Mod num_pts)
            pt_after2 = pts((i + 2) Mod num_pts)

            p1 = pt
            p4 = pt_after

            Di.X = pt_after.X - pt_before.X
            Di.Y = pt_after.Y - pt_before.Y
            p2.X = pt.X + control_scale * Di.X
            p2.Y = pt.Y + control_scale * Di.Y

            DiPlus1.X = pt_after2.X - pts(i).X
            DiPlus1.Y = pt_after2.Y - pts(i).Y
            p3.X = pt_after.X - control_scale * DiPlus1.X
            p3.Y = pt_after.Y - control_scale * DiPlus1.Y

            DrawBezier(gr, the_pen, dt, p1, p2, p3, p4)
        Next i
    End Sub
End Module
